/*
  airbox-dss.js
  Angular service that provides airbox-addon-specific functions
*/

var dstinydss = angular.module('dSTinyDSS', ['dSS']);


dstinydss.service('dSTinyDSS', ['$q', 'dss', function($q, dss) {
	var rootpath = "/scripts/wesco-balance";
	var service = this;

	var scenenumbers = [0, 5, 17, 18, 19];

	// When reading stringified objects (JSON) from the property tree, string has to be decoded before calling JSON.parse
	function HTMLDecode(string) {
		return string.replace(/&amp;/g, '&').replace(/&gt;/g, '>').replace(/&lt;/g, '<').replace(/&quot;/g, '"');
	}

	// Get list of registered targets for push notifcations. The background script keeps this list up-to-date.
	this.getPushTargetOptions = function() {
		return dss.Property.getString(rootpath + '/Notifications/PushTargetOptions').then(function(result) {
			return JSON.parse(HTMLDecode(result));
		});
	}

	// Get list of devices that can be selected as exhaust flaps
	/*
	  GTIN			Device
	  4290046000959	SW-KL200
	*/
	this.getExhaustFlapCandidates = function() {
		return dss.Apartment.getDevices().then(function(devices) {
			var candidates = [];
			for (var i = 0; i < devices.length; i++) {
				if (devices[i].isPresent && devices[i].GTIN == "4290046000959") {
					candidates.push({
						'value': devices[i].id,
						'name': (devices[i].name ? devices[i].name : devices[i].id) + ' (' + devices[i].hwInfo + ')',
					});
				}
			}
			return candidates;
		});
	};

	// Set list of push targets (array with device ids) that get notifications
	this.setPushTargets = function(targets) {
		return dss.Property.setString(rootpath + '/Notifications/PushTargets', JSON.stringify(targets));
	};

	this.setBaseVentilation = function(enabled, level) {
		var promises = [];
		promises.push(dss.Property.setString(rootpath + '/Controller/BaseVentilation/Level', level));
		promises.push(dss.Property.setBoolean(rootpath + '/Controller/BaseVentilation/Enabled', enabled));

		return $q.all(promises);
	};

	this.setAirboxVolumeLevels = function(levels) {
		var l = JSON.stringify(levels);
		return dss.Property.setString(rootpath + '/Controller/airbox_volume_levels', l);
	};

	this.setExHoodVolumeLevels = function(levels) {
		var l = JSON.stringify(levels);
		return dss.Property.setString(rootpath + '/Controller/exhood_volume_levels', l);
	};
	// Set dsid of exhaust flap device
	this.setExhaustFlap = function(dsid) {
		return dss.Property.setString(rootpath + '/ExhaustFlap', dsid);
	};

	this.setEmailAddress = function(email) {
		return dss.Property.setString(rootpath + '/Notifications/EmailAddress', email);
	};

	this.getCurrentLevel = function() {
		return dss.Property.getInteger(rootpath + '/Controller/CurrentLevel');
	};

	this.getExhoodCurrentValue_fan = function() {
		return dss.Property.getInteger(rootpath + '/Controller/ExhoodCurrentValue_fan');
	};
	this.getExhoodCurrentValue_flap = function() {
		return dss.Property.getInteger(rootpath + '/Controller/ExhoodCurrentValue_flap');
	};
	this.getBusyFlag = function() {
		try {
			return dss.Property.getBoolean(rootpath + '/Controller/writingInProgress');
		} catch (err) {
			return false;
		}
	};

	this.getPresenceMode = function() {
		try {
			return dss.Property.getString(rootpath + '/Controller/PresenceState_current');
		} catch (err) {
			return undefined;
		}
	};

	this.getBigMode = function() {
		try {
			return dss.Property.getString(rootpath + '/Controller/BigModeState_current');
		} catch (err) {
			return undefined;
		}
	};


	this.getDayFromVentilationSchedule = function(day) {

		var rule = {};
		var rules = [];
		return dss.Property.getString(rootpath + '/Controller/VentilationSchedule/' + day).then(function(s) {
			if (s !== undefined && s !== null) {
				s = HTMLDecode(s);
				var dict_day = JSON.parse(s);
				for (var key in dict_day) {
					var dict_h = dict_day[key];
					rule = {};
					rule['day'] = day;

					if (dict_h['Presence'] !== undefined && dict_h['Presence'] !== null) {
						rule['Presence'] = dict_h['Presence'];
					}
					if (dict_h['Absence'] !== undefined && dict_h['Absence'] !== null) {
						rule['Absence'] = dict_h['Absence'];
					}
					if (dict_h['from'] !== undefined && dict_h['from'] !== null) {
						rule['from'] = dict_h['from'];
					}
					if (dict_h['until'] !== undefined && dict_h['until'] !== null) {
						rule['until'] = dict_h['until'];
					}

					rules.push(rule);
				}
			}
			return rules;
		}, function error() {
			return $q.reject();
		})
	};

	this.getVentilationSchedule = function() {
		var all_rules = [];

		return dss.Property.getChildren(rootpath + '/Controller/VentilationSchedule/')
			.then(function(devices) {
				var promises = devices.map(function(device) {
					return service.getDayFromVentilationSchedule(device.name).then(function(rules) {
						for (var rule in rules) {
							all_rules.push(rules[rule]);
						}
					});
				});
				return $q.all(promises).then(function() {
					return all_rules;
				});
			}).catch(function(devices) {
				return all_rules; // return empty list
			});
	};


	this.writeWCFanRegs = function(dsuid, dict) {
		return dss.Event.raise('wesco_UI', {
			'action': 'SetWCFanRegs',
			'dsuid': dsuid,
			"Regs": JSON.stringify(dict),
		});
	};


	this.setLevel = function(level) {
		return dss.Event.raise('wesco_UI', {
			'action': 'SetAirboxesLevel',
			'level': level,
		});
		//return dss.Apartment.callScene(scenenumbers[level],groupid);
	};

	this.rediscoverAirboxes = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'RediscoverDevices_Airboxes',
		});
	};

	this.rediscoverExhoods = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'RediscoverDevices_Exhoods',
		});
	};
	this.rediscoverWCFans = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'RediscoverDevices_WCFans',
		});
	};



	this.refreshPushTargetOptions = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'RefreshPushTargetOptions',
		});
	};

	this.readRegs = function(dsid, regs, type) {
		return dss.Event.raise('wesco_UI', {
			'action': 'TinyRegs',
			'type': type,
			'subaction': 'readSingle',
			'dSID': dsid,
			'args': JSON.stringify(regs),
		});
	};

	this.writeRegs = function(dsid, regs, type) {
		return dss.Event.raise('wesco_UI', {
			'action': 'TinyRegs',
			'type': type,
			'subaction': 'writeSingle',
			'dSID': dsid,
			'args': JSON.stringify(regs),
		});
	};

	this.writeRegs_toAll = function(dsid, regs, type) {
		return dss.Event.raise('wesco_UI', {
			'action': 'TinyRegs',
			'type': type,
			'subaction': 'writeAll',
			'args': JSON.stringify(regs),
		});
	};


	this.storeConfigurableParameters = function() { //@Diego
		return dss.Event.raise('wesco_UI', {
			'action': 'StoredConfigurableParameters',
		});
	};

	this.storeExhaustFlap = function() { //@Diego
		return dss.Event.raise('wesco_UI', {
			'action': 'StoredExhaustFlap',
		});
	};

	this.restartAirboxes = function() { //@Diego
		return dss.Event.raise('wesco_UI', {
			'action': 'SetAirboxesLevel',
			'level': '2',
		});
	};

	this.configVentilationSchedule = function(schedule) {
		return dss.Event.raise('wesco_UI', {
			'action': 'configVentilationSchedule',
			'schedule': JSON.stringify(schedule),
		});
	};


	this.clearAllAirboxErrors = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'ClearAllAirboxErrors',
		});
	};

	this.clearAllAirboxErrors = function() {
		return dss.Event.raise('wesco_UI', {
			'action': 'ClearAllAirboxErrors',
		});
	};

	this.clearAirboxError = function(dSUID) {
		return dss.Event.raise('wesco_UI', {
			'action': 'ClearAirboxError',
			'dSUID': dSUID,
		});
	};

	this.getErrorLog = function() {
		return dss.getLogfile('/dss/js/wesco.log');
	};

	this.getConfig = function() {
		var result = {};
		var promises = [];
		promises.push(dss.Property.getString(rootpath + '/Notifications/PushTargets').then(function(value) {
			result['PushTargets'] = JSON.parse(HTMLDecode(value));
		}));
		promises.push(dss.Property.getString(rootpath + '/Notifications/EmailAddress').then(function(value) {
			result['EmailAddress'] = value;
		}));
		promises.push(dss.Property.getString(rootpath + '/ExhaustFlap').then(function(value) {
			result['ExhaustFlap'] = value;
		}));
		promises.push(dss.Property.getInteger(rootpath + '/Controller/ClusterID').then(function(value) {
			result['ClusterID'] = value;
		}));

		promises.push(dss.Property.getBoolean(rootpath + '/Controller/BaseVentilation/Enabled').then(function(value) {
			result['BaseVentilationEnabled'] = value;
		}));



		promises.push(dss.Property.getString(rootpath + '/Controller/BaseVentilation/Level').then(function(value) {
			result['BaseVentilationLevel'] = value;

			var levels = {
				'0': "Off",
				'1': "Low",
				'2': "Medium",
				'3': "Nominal",
				'4': "Full",
			};
			result['BaseVentilationLevelValues'] = levels;


		}));



		return $q.all(promises).then(function() {
			return result;
		});
	};

	this.getAirboxConfig = function() {
		var result = {};
		var promises = [];
		promises.push(dss.Property.getInteger(rootpath + '/Airboxes/FilterReplacementInterval').then(function(value) {
			result['FilterReplacementInterval'] = value;
		}));
		return $q.all(promises).then(function() {
			return result;
		});
	};

	// Get info for a single airbox
	this.getAirbox = function(deviceid) {
		var result = {};
		var promises = [];

		result['dSUID'] = deviceid; // added variable dsuid (which is different than the dsid)!

		promises.push(dss.Property.getString(rootpath + '/Airboxes/Devices/' + deviceid + '/dSID').then(function(value) {
			result['dSID'] = value;
			return dss.Device.getName(result['dSID']).then(function(name) {
				result['Description'] = name;
			});
		}, function error() {
			result = {};
			return $q.reject();
		}));

		promises.push(dss.Property.getString(rootpath + '/Airboxes/Devices/' + deviceid + '/errorCode').then(function(value) {
			result['errorCode'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorCode");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/Airboxes/Devices/' + deviceid + '/Status').then(function(value) {

			result['Status'] = "NORMAL";

			if (value & 0x01) {
				result['Status'] = result['Status'] + "LOCKED";
			}
			if (value & 0x40) {
				result['Status'] = result['Status'] + "FAN_MISSING";
			}
			// console.log(value);
			// console.log(result['Status']);
		}, function error() {
			result = {};
			return $q.reject("Field not found: Status");
		}));

		promises.push(dss.Property.getString(rootpath + '/Airboxes/Devices/' + deviceid + '/errorTime').then(function(value) {
			result['errorTime'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorTime");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/Airboxes/Devices/' + deviceid + '/errorCounter').then(function(value) {
			result['errorCounter'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorCounter");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/Airboxes/Devices/' + deviceid + '/SoftwareVersion').then(function(value) {
			result['SoftwareVersion'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorCounter");
		}));

		return $q.all(promises).then(function() {
			return result;
		}, function error() {
			return $q.reject();
		});

	};

	this.getAirboxVolumeLevels = function() {
		return dss.Property.getString(rootpath + '/Controller/airbox_volume_levels').then(function(value) {
			return JSON.parse(HTMLDecode(value));
		});
	};

	this.getExHoodVolumeLevels = function() {
		return dss.Property.getString(rootpath + '/Controller/exhood_volume_levels').then(function(value) {
			return JSON.parse(HTMLDecode(value));
		});
	};


	// Get info for all configured airboxes
	this.getAirboxes = function() {
		var listReady = dss.Property.getChildren(rootpath + '/Airboxes/Devices')
			.then(function(devices) {
				var devInfoPromises = devices.map(function(device) {
					return service.getAirbox(device.name);
				});

				return $q.all(devInfoPromises);
			}).catch(function() {
				return [];
			});
		return listReady;
	};


	// Get info for a single wc fan
	this.getWCFan = function(deviceid) {
		var result = {};
		var promises = [];

		result['dSUID'] = deviceid; // added variable dsuid (which is different than the dsid)!

		promises.push(dss.Property.getString(rootpath + '/WCFans/Devices/' + deviceid + '/dSID').then(function(value) {
			result['dSID'] = value;
			return dss.Device.getName(result['dSID']).then(function(name) {
				result['Description'] = name;
			});
		}, function error() {
			result = {};
			return $q.reject();
		}));


		promises.push(dss.Property.getInteger(rootpath + '/WCFans/Devices/' + deviceid + '/ZoneID').then(function(value) {
			result['ZoneID'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: ZoneID");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/WCFans/Devices/' + deviceid + '/Status').then(function(value) {
			result['Status'] = "NORMAL";

			// console.log(value);
			// console.log(result['Status']);
		}, function error() {
			result = {};
			return $q.reject("Field not found: Status");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/WCFans/Devices/' + deviceid + '/VolumeRate').then(function(value) {
			result['VolumeRate'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: VolumeRate");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/WCFans/Devices/' + deviceid + '/OnTime').then(function(value) {
			result['OnTime'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: OnTime");
		}));


		return $q.all(promises).then(function() {
			return result;
		}, function error() {
			return $q.reject();
		});

	};


	// Get info for all configured airboxes
	this.getWCFans = function() {
		var listReady = dss.Property.getChildren(rootpath + '/WCFans/Devices')
			.then(function(devices) {
				var devInfoPromises = devices.map(function(device) {
					return service.getWCFan(device.name);
				});
				return $q.all(devInfoPromises);
			}).catch(function() {
				return [];
			});
		return listReady;
	};



	// Get info for a single exhood -> in theory only one should be present
	this.getExHood = function(deviceid) {
		var result = {};
		var promises = [];

		result['dSUID'] = deviceid; // added variable dsuid (which is different than the dsid)!

		promises.push(dss.Property.getString(rootpath + '/ExHood/Devices/' + deviceid + '/Type').then(function(value) {
			result['Type'] = value; // % value of the flap
		}, function error() {
			result = {};
			return $q.reject("Field not found: Type");
		}));


		promises.push(dss.Property.getString(rootpath + '/ExHood/Devices/' + deviceid + '/dSID').then(function(value) {
			result['dSID'] = value;
			return dss.Device.getName(result['dSID']).then(function(name) {
				result['Description'] = name;
			});
		}, function error() {
			result = {};
			return $q.reject();
		}));

		promises.push(dss.Property.getString(rootpath + '/ExHood/Devices/' + deviceid + '/errorCode').then(function(value) {
			result['errorCode'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorCode");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/ExHood/Devices/' + deviceid + '/Status').then(function(value) {

			result['Status'] = "NORMAL";

			// console.log(value);
			// console.log(result['Status']);
		}, function error() {
			result = {};
			return $q.reject("Field not found: Status");
		}));

		promises.push(dss.Property.getString(rootpath + '/ExHood/Devices/' + deviceid + '/errorTime').then(function(value) {
			result['errorTime'] = value;
		}, function error() {
			result = {};
			return $q.reject("Field not found: errorTime");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/Controller/ExhoodCurrentValue_fan').then(function(value) {
			result['CurrentValue_fan'] = value; // % value of the fan
		}, function error() {
			result = {};
			return $q.reject("Field not found: CurrentValue_fan");
		}).catch(function() {
			result = {};
			return $q.reject("Field not found: CurrentValue_fan");
		}));

		promises.push(dss.Property.getInteger(rootpath + '/Controller/ExhoodCurrentValue_flap').then(function(value) {
			result['CurrentValue_flap'] = value; // % value of the fan
		}, function error() {
			result = {};
			return $q.reject("Field not found: CurrentValue_flap");
		}).catch(function() {
			result = {};
			return $q.reject("Field not found: CurrentValue_flap");

		}));


		return $q.all(promises).then(function() {
			return result;
		}, function error() {
			return $q.reject();
		});

	};

	// Get info for all configured Exhoods -> in theory should be no more than one
	this.getExHoods = function() {
		var listReady = dss.Property.getChildren(rootpath + '/ExHood/Devices')
			.then(function(devices) {
				var devInfoPromises = devices.map(function(device) {
					return service.getExHood(device.name);
				});
				return $q.all(devInfoPromises);
			}).catch(function() {
				return [];
			});
		return listReady;
	};



	this.parseVarFile = function(contents_raw) {
		var result = [];
		var contents = contents_raw.replace(/(\rn|\r|\n)/g, '\n'); //replace end of lines of (windows or mac by unix-like)
		var lines = contents.split(/\n/);
		for (var i = 0; i < lines.length; i++) {
			if (!/\S/.test(lines[i])) {
				continue;
			}
			if (/#/.test(lines[i])) { // skip commentaren
				continue;
			}

			var data = lines[i].split(/\t/);
			result.push({
				'reg': data[0],
				'value': data[1],
			});
		}
		return result;
	};

	this.parseExHoodVarFile = function(contents_raw) {
		var result = [];
		var contents = contents_raw.replace(/(\rn|\r|\n)/g, '\n'); //replace end of lines of (windows or mac by unix-like)
		var lines = contents.split(/\n/);
		for (var i = 0; i < lines.length; i++) {
			if (!/\S/.test(lines[i])) {
				continue;
			}
			if (/#/.test(lines[i])) { // skip commentaren
				continue;
			}

			var data = lines[i].split(/\t/);
			result.push({
				'reg': data[0],
				'value': data[1],
				'value2': data[2],
			});
		}
		return result;
	};

	this.parseVentilationScheduleFile = function(contents_raw) {
		var result = [];
		var contents = contents_raw.replace(/(\rn|\r|\n)/g, '\n'); //replace end of lines of (windows or mac by unix-like)
		var lines = contents.split(/\n/);
		for (var i = 0; i < lines.length; i++) {
			if (!/\S/.test(lines[i])) {
				continue;
			}
			if (/#/.test(lines[i])) { // skip commentaren
				continue;
			}

			var data = lines[i].split(/\t/);
			result.push({
				'day': data[0],
				'from': data[1],
				'until': data[2],
				'presence': data[3],
				'absence': data[4],
			});
		}
		return result;
	};


}]);